require "koho/version"
require 'httparty'
require 'json'

KOHO_API_CLIENT_ENVIRONMENT ||= :development #:production

module Koho

  class Client
    include HTTParty

    if KOHO_API_CLIENT_ENVIRONMENT == :production
      API_BASE_URL = 'https://stg.koho-online.com/api'
    else
      API_BASE_URL = 'https://dev.koho-online.com/api'
    end

    base_uri API_BASE_URL
      
    attr_accessor :company_id,
                  :token

    def initialize company_id, token
      @company_id = company_id
      @token = token
    end

    def get action, data = {}
      res = (self.class.get action, query: data.merge(company_id: @company_id, token: @token)).response.body
      JSON.parse(res)
    end

    def post action, data = {}
      JSON.parse((self.class.post action, {method: '_POST', body: {company_id: @company_id, token: @token}.merge(data)}).response.body)
    end

    def put action, data = {}
      JSON.parse((self.class.put action, {method: '_PUT', body: {company_id: @company_id, token: @token}.merge(data)}).response.body)
    end

    def delete action
      JSON.parse((self.class.delete action, {method: '_DELETE', body: {company_id: @company_id, token: @token}}).response.body)
    end

    def customers; Customers.new company_id, token; end
    def invoices; Invoices.new company_id, token; end
    def inbound_invoices; InboundInvoices.new company_id, token; end
    def invoice_terms; InvoiceTerms.new company_id, token; end
    def contracts; Contracts.new company_id, token; end
    def projects; Projects.new company_id, token; end
    def project_tasks; ProjectTasks.new company_id, token; end
    def work_sessions; WorkSessions.new company_id, token; end

    ##########################3

    def info 
      get '/info'
    end

    class InboundInvoices< Koho::Client
      
      base_uri  API_BASE_URL+'/invoice_inbounds'


      def list
        get '/'
      end

      def find id
        get "/#{id}"
      end


      def create params
        post '/', invoice_inbound: params
      end

      def update id, params
        put "/#{id}", invoice_inbound: params
      end

      def destroy id
        delete "/#{id}"
      end
      

    end

    class Invoices < Koho::Client
      
      base_uri  API_BASE_URL+'/invoices'


      def list
        get '/'
      end

      def find id
        get "/#{id}"
      end

      def find_by_contract contract_id
        get "/find_by_contract/#{contract_id}"
      end

      def find_by_customer customer_id
        get "/find_by_customer/#{customer_id}"
      end

      def create params
        post '/', invoice: params
      end

      def update id, params
        put "/#{id}", invoice: params
      end

      def destroy id
        delete "/#{id}"
      end
      

    end

    class ProjectTasks < Koho::Client
      
      base_uri  API_BASE_URL+'/project_tasks'


      def list
        get '/'
      end

      def find id
        get "/#{id}"
      end

      def  find_by_project project_id
        get "/find_by_project/#{project_id}"
      end

    end

    class Projects < Koho::Client
      
      base_uri  API_BASE_URL+'/projects'


      def list
        get '/'
      end

      def find id
        get "/#{id}"
      end

      def find_by_customer customer_id
        get "/find_by_customer/#{customer_id}"
      end

      def create params
        post '/', customer: params
      end

      def update id, params
        put "/#{id}", customer: params
      end

      def destroy id
        delete "/#{id}"
      end
      
      def permitted_projects
        get "/permitted_projects"
      end

    end


    class WorkSessions < Koho::Client
      
      base_uri  API_BASE_URL+'/work_sessions'


      def list opts={}
        get "/?#{opts.to_query}"
      end

      def find id
        get "/#{id}"
      end

      def create params
        post '/', work_session: params
      end

      def update id, params
        put "/#{id}", work_session: params
      end

      def destroy id
        delete "/#{id}"
      end

    end


    class Customers < Koho::Client
      
      base_uri  API_BASE_URL+'/customers'


      def list
        get '/'
      end

      def find id
        get "/#{id}"
      end

      def create params
        post '/', customer: params
      end

      def update id, params
        put "/#{id}", customer: params
      end

      def destroy id
        delete "/#{id}"
      end

      def list_with_permissions
        get '/permitted'
      end
      

    end


    class InvoiceTerms < Koho::Client
      base_uri  API_BASE_URL+'/invoice_terms'

      def find_by_contract contract_id
        get "/find_by_contract/#{contract_id}"
      end

      def find_by_contract_and_date contract_id, date
        get "/find_by_contract_and_date/#{contract_id}/#{date.to_s}"
      end

      def mark_non_invoicable id
        post "/#{id}/mark_non_invoicable" 
      end

      def mark_invoicable id
        post "/#{id}/mark_invoicable" 
      end
    end

    class Contracts < Koho::Client
      
      base_uri  API_BASE_URL+'/contracts'


      def list
        get '/'
      end

      def invoice_check
        status = nil
        Thread.new{status = post '/invoice_check'}
        sleep 5
        return status || {status: :ok, message: 'Request sent'}
      end

      def find id
        get "/#{id}"
      end

      def find_by_customer customer_id
        get "/find_by_customer/#{customer_id}"
      end
       
       #...contracts.create name: 'Testisopimus', confirmed_by_manager: true, customer_id: XXX, description: 'Uintipaikka ja aika', start_date: '10.11.2013', products_attributes: [{product_type_id: YYY, count: 2}], acceptance_attributes: {verification_level: :signature}


      def create params
        post '/', contract: params
      end

      def update id, params
        put "/#{id}", contract: params
      end

      def destroy id
        delete "/#{id}"
      end

    end

  end
end

